@@ -4,8 +4,7 @@
 KW_DT = '_dt'
 KW_T = '_t'
 KW_I = '_i'
-ARG_KEYWORDS = ['_dt', '_t', '_i',
-                '_obj_i', '_pre_i', '_post_i']
+ARG_KEYWORDS = ['_dt', '_t', '_i', '_obj_i', '_pre_i', '_post_i']
 
 # name of the neuron group
 NEU_GROUP_TYPE = 'NeuGroup'
