@@ -1,21 +1,16 @@
 # -*- coding: utf-8 -*-
 
 import re
-import typing
 
 import numpy as np
 
+from . import base
 from . import constants
-from .base import Ensemble
-from .base import ObjType
-from .neurons import NeuGroup
-from .neurons import NeuSubGroup
-from .types import SynState
+from . import neurons
+from .. import connectivity
+from .. import errors
 from .. import profile
 from .. import tools
-from ..connectivity import Connector
-from ..errors import ModelDefError
-from ..errors import ModelUseError
 
 __all__ = [
     'SynType',
@@ -26,33 +21,22 @@ __all__ = [
 _SYN_CONN_NO = 0
 
 
-class SynType(ObjType):
+class SynType(base.ObjType):
     """Abstract Synapse Type.
 
     It can be defined based on a collection of synapses or a single synapse model.
     """
 
-    def __init__(
-            self,
-            name: str,
-            ST: SynState,
-            steps: typing.Union[callable, list, tuple],
-            mode: str = 'vector',
-            requires: dict = None,
-            hand_overs: typing.Dict = None,
-            heter_params_replace: dict = None,
-    ):
+    def __init__(self, name, ST, steps, mode='vector', requires=None, hand_overs=None, ):
         if mode not in [constants.SCALAR_MODE, constants.VECTOR_MODE, constants.MATRIX_MODE]:
-            raise ModelDefError('SynType only support "scalar", "vector" or "matrix".')
+            raise errors.ModelDefError('SynType only support "scalar", "vector" or "matrix".')
 
-        super(SynType, self).__init__(
-            ST=ST,
-            requires=requires,
-            steps=steps,
-            name=name,
-            mode=mode,
-            heter_params_replace=heter_params_replace,
-            hand_overs=hand_overs)
+        super(SynType, self).__init__(ST=ST,
+                                      requires=requires,
+                                      steps=steps,
+                                      name=name,
+                                      mode=mode,
+                                      hand_overs=hand_overs)
 
         # inspect delay keys
         # ------------------
@@ -70,50 +54,40 @@ class SynType(ObjType):
             _delay_keys = set()
             delay_keys_in_left = set(re.findall(r'ST\[[\'"](\w+)[\'"]\]', delay_func_code_left))
             if len(delay_keys_in_left) > 0:
-                raise ModelDefError(f'Delayed function cannot assign value to "ST".')
+                raise errors.ModelDefError(f'Delayed function cannot assign value to "ST".')
             delay_keys = set(re.findall(r'ST\[[\'"](\w+)[\'"]\]', delay_func_code))
             if len(delay_keys) > 0:
                 _delay_keys.update(delay_keys)
             self._delay_keys = list(_delay_keys)
 
 
-class SynConn(Ensemble):
+class SynConn(base.Ensemble):
     """Synaptic connections.
 
     Parameters
     ----------
     model : SynType
         The instantiated neuron type model.
-    pars_update : dict, None
+    pars_update : dict
         Parameters to update.
-    pre_group : NeuGroup, None
+    pre_group : neurons.NeuGroup, neurons.NeuSubGroup
         Pre-synaptic neuron group.
-    post_group : NeuGroup, None
+    post_group : neurons.NeuGroup, neurons.NeuSubGroup
         Post-synaptic neuron group.
-    conn : Connector, None
+    conn : connectivity.Connector
         Connection method to create synaptic connectivity.
     num : int
         The number of the synapses.
     delay : float
         The time of the synaptic delay.
-    monitors : list, tuple, None
+    monitors : list, tuple
         Variables to monitor.
-    name : str, None
+    name : str
         The name of the neuron group.
     """
 
-    def __init__(
-            self,
-            model: SynType,
-            pre_group: typing.Union[NeuGroup, NeuSubGroup] = None,
-            post_group: typing.Union[NeuGroup, NeuSubGroup] = None,
-            conn: typing.Union[Connector, np.ndarray, typing.Dict] = None,
-            delay: float = 0.,
-            name: str = None,
-            monitors: typing.Union[typing.Tuple, typing.List] = None,
-            satisfies: typing.Dict = None,
-            pars_update: typing.Dict = None,
-    ):
+    def __init__(self, model, pre_group=None, post_group=None, conn=None, delay=0.,
+                 name=None, monitors=None, satisfies=None, pars_update=None, ):
         # name
         # ----
         if name is None:
@@ -126,13 +100,13 @@ class SynConn(Ensemble):
         # model
         # ------
         if not isinstance(model, SynType):
-            raise ModelUseError(f'{type(self).__name__} receives an instance of {SynType.__name__}, '
-                                f'not {type(model).__name__}.')
+            raise errors.ModelUseError(f'{type(self).__name__} receives an instance of {SynType.__name__}, '
+                                       f'not {type(model).__name__}.')
 
         if model.mode == 'scalar':
             if pre_group is None or post_group is None:
-                raise ModelUseError('Using scalar-based synapse model must '
-                                    'provide "pre_group" and "post_group".')
+                raise errors.ModelUseError('Using scalar-based synapse model must '
+                                           'provide "pre_group" and "post_group".')
 
         # pre or post neuron group
         # ------------------------
@@ -143,10 +117,10 @@ class SynConn(Ensemble):
         if pre_group is not None and post_group is not None:
             # check
             # ------
-            if not isinstance(pre_group, (NeuGroup, NeuSubGroup)):
-                raise ModelUseError('"pre_group" must be an instance of NeuGroup/NeuSubGroup.')
-            if not isinstance(post_group, (NeuGroup, NeuSubGroup)):
-                raise ModelUseError('"post_group" must be an instance of NeuGroup/NeuSubGroup.')
+            if not isinstance(pre_group, (neurons.NeuGroup, neurons.NeuSubGroup)):
+                raise errors.ModelUseError('"pre_group" must be an instance of NeuGroup/NeuSubGroup.')
+            if not isinstance(post_group, (neurons.NeuGroup, neurons.NeuSubGroup)):
+                raise errors.ModelUseError('"post_group" must be an instance of NeuGroup/NeuSubGroup.')
 
             # pre and post synaptic state
             self.pre = pre_group.ST
@@ -155,32 +129,33 @@ class SynConn(Ensemble):
             if conn is not None:
                 # connections
                 # ------------
-                if isinstance(conn, Connector):
+                if isinstance(conn, connectivity.Connector):
                     self.conn = conn
                     self.conn(pre_group.indices, post_group.indices)
                 else:
                     if isinstance(conn, np.ndarray):
                         # check matrix dimension
                         if np.ndim(conn) != 2:
-                            raise ModelUseError(f'"conn" must be a 2D array, not {np.ndim(conn)}D.')
+                            raise errors.ModelUseError(f'"conn" must be a 2D array, not {np.ndim(conn)}D.')
                         # check matrix shape
                         conn_shape = np.shape(conn)
                         if not (conn_shape[0] == pre_group.num and conn_shape[1] == post_group.num):
-                            raise ModelUseError(f'The shape of "conn" must be ({pre_group.num}, {post_group.num})')
+                            raise errors.ModelUseError(
+                                f'The shape of "conn" must be ({pre_group.num}, {post_group.num})')
                         # get pre_ids and post_ids
                         pre_ids, post_ids = np.where(conn > 0)
                     else:
                         # check conn type
                         if not isinstance(conn, dict):
-                            raise ModelUseError(f'"conn" only support "dict", 2D ndarray, '
-                                                f'or instance of bp.connect.Connector.')
+                            raise errors.ModelUseError(f'"conn" only support "dict", 2D ndarray, '
+                                                       f'or instance of bp.connect.Connector.')
                         # check conn content
                         if not ('i' in conn and 'j' in conn):
-                            raise ModelUseError('When provided "conn" is a dict, "i" and "j" must in "conn".')
+                            raise errors.ModelUseError('When provided "conn" is a dict, "i" and "j" must in "conn".')
                         # get pre_ids and post_ids
                         pre_ids = np.asarray(conn['i'], dtype=np.int_)
                         post_ids = np.asarray(conn['j'], dtype=np.int_)
-                    self.conn = Connector()
+                    self.conn = connectivity.Connector()
                     self.conn.pre_ids = pre_group.indices.flatten()[pre_ids]
                     self.conn.post_ids = post_group.indices.flatten()[post_ids]
 
@@ -202,7 +177,7 @@ class SynConn(Ensemble):
         try:
             assert 0 < num < 2 ** 64
         except AssertionError:
-            raise ModelUseError('Total synapse number "num" must be a valid number in "uint64".')
+            raise errors.ModelUseError('Total synapse number "num" must be a valid number in "uint64".')
 
         # initialize
         # ----------
@@ -232,14 +207,14 @@ class SynConn(Ensemble):
         if self.model.mode == constants.MATRIX_MODE:
             if pre_group is None:
                 if 'pre_size' not in satisfies:
-                    raise ModelUseError('"pre_size" must be provided in "satisfies" when "pre_group" is none.')
+                    raise errors.ModelUseError('"pre_size" must be provided in "satisfies" when "pre_group" is none.')
                 pre_size = satisfies['pre_size']
             else:
                 pre_size = pre_group.size
 
             if post_group is None:
                 if 'post_size' not in satisfies:
-                    raise ModelUseError('"post_size" must be provided in "satisfies" when "post_group" is none.')
+                    raise errors.ModelUseError('"post_size" must be provided in "satisfies" when "post_group" is none.')
                 post_size = satisfies['post_size']
             else:
                 post_size = post_group.size
