@@ -11,6 +11,8 @@ from ..errors import CodeError
 from ..errors import DiffEquationError
 
 __all__ = [
+    'NoiseHandler',
+
     'CodeLineFormatter',
     'format_code',
 
@@ -24,10 +26,6 @@ __all__ = [
     'replace_func',
     'FuncCallFinder',
 
-    # analyse differential equations
-    'analyse_diff_eq',
-    'DiffEquationAnalyser',
-
     # string processing
     'get_identifiers',
     'get_main_code',
@@ -91,6 +89,7 @@ def get_identifiers(expr, include_numbers=False):
     >>> print(sorted(list(ids)))
     ['.3e-10', '17', '3', '8', 'A', '_b', 'a', 'c5', 'f', 'tau_2']
     """
+
     _ID_KEYWORDS = {'and', 'or', 'not', 'True', 'False'}
     identifiers = set(re.findall(r'\b[A-Za-z_][A-Za-z0-9_.]*\b', expr))
     # identifiers = set(re.findall(r'\b[A-Za-z_][.?[A-Za-z0-9_]*]*\b', expr))
@@ -103,139 +102,21 @@ def get_identifiers(expr, include_numbers=False):
     return (identifiers - _ID_KEYWORDS) | numbers
 
 
-class DiffEquationAnalyser(ast.NodeTransformer):
-    expression_ops = {
-        'Add': '+', 'Sub': '-', 'Mult': '*', 'Div': '/',
-        'Mod': '%', 'Pow': '**', 'BitXor': '^', 'BitAnd': '&',
-    }
+class NoiseHandler(object):
+    normal_pattern = re.compile(r'(_normal_like_)\((\w+)\)')
 
-    def __init__(self):
-        self.variables = []
-        self.expressions = []
-        self.f_expr = None
-        self.g_expr = None
-        self.returns = []
-        self.return_type = None
+    @staticmethod
+    def vector_replace_f(m):
+        return 'numpy.random.normal(0., 1., ' + m.group(2) + '.shape)'
 
-    # TODO : Multiple assignment like "a = b = 1" or "a, b = f()"
-    def visit_Assign(self, node):
-        targets = node.targets
-        try:
-            assert len(targets) == 1
-        except AssertionError:
-            raise DiffEquationError('BrainPy currently does not support multiple '
-                                    'assignment in differential equation.')
-        self.variables.append(targets[0].id)
-        self.expressions.append(ast2code(ast.fix_missing_locations(node.value)))
-        return node
+    @staticmethod
+    def scalar_replace_f(m):
+        return 'numpy.random.normal(0., 1.)'
 
-    def visit_AugAssign(self, node):
-        var = node.target.id
-        self.variables.append(var)
-        op = ast2code(ast.fix_missing_locations(node.op))
-        expr = ast2code(ast.fix_missing_locations(node.value))
-        self.expressions.append(f"{var} {op} {expr}")
-        return node
+    @staticmethod
+    def cuda_replace_f(m):
+        return 'xoroshiro128p_normal_float64(rng_states, _obj_i)'
 
-    def visit_AnnAssign(self, node):
-        raise DiffEquationError('Do not support an assignment with a type annotation.')
-
-    def visit_Return(self, node):
-        value = node.value
-        if isinstance(value, (ast.Tuple, ast.List)):  # a tuple/list return
-            v0 = value.elts[0]
-            if isinstance(v0, (ast.Tuple, ast.List)):  # item 0 is a tuple/list
-                # f expression
-                if isinstance(v0.elts[0], ast.Name):
-                    self.f_expr = ('_f_res_', v0.elts[0].id)
-                else:
-                    self.f_expr = ('_f_res_', ast2code(ast.fix_missing_locations(v0.elts[0])))
-
-                if len(v0.elts) == 1:
-                    self.return_type = '(x,),'
-                elif len(v0.elts) == 2:
-                    self.return_type = '(x,x),'
-                    # g expression
-                    if isinstance(v0.elts[1], ast.Name):
-                        self.g_expr = ('_g_res_', v0.elts[1].id)
-                    else:
-                        self.g_expr = ('_g_res_', ast2code(ast.fix_missing_locations(v0.elts[1])))
-                else:
-                    raise DiffEquationError(f'The dxdt should have the format of (f, g), not '
-                                            f'"({ast2code(ast.fix_missing_locations(v0.elts))})"')
-
-                # returns
-                for i, item in enumerate(value.elts[1:]):
-                    if isinstance(item, ast.Name):
-                        self.returns.append(item.id)
-                    else:
-                        self.returns.append(ast2code(ast.fix_missing_locations(item)))
-
-            else:  # item 0 is not a tuple/list
-                # f expression
-                if isinstance(v0, ast.Name):
-                    self.f_expr = ('_f_res_', v0.id)
-                else:
-                    self.f_expr = ('_f_res_', ast2code(ast.fix_missing_locations(v0)))
-
-                if len(value.elts) == 1:
-                    self.return_type = 'x,'
-                elif len(value.elts) == 2:
-                    self.return_type = 'x,x'
-                    # g expression
-                    if isinstance(value.elts[1], ast.Name):
-                        self.g_expr = ('_g_res_', value.elts[1].id)
-                    else:
-                        self.g_expr = ("_g_res_", ast2code(ast.fix_missing_locations(value.elts[1])))
-                else:
-                    raise DiffEquationError('Cannot parse return expression. It should have the '
-                                            'format of "(f, [g]), [return values]"')
-        else:
-            self.return_type = 'x'
-            if isinstance(value, ast.Name):  # a name return
-                self.f_expr = ('_f_res_', value.id)
-            else:  # an expression return
-                self.f_expr = ('_f_res_', ast2code(ast.fix_missing_locations(value)))
-        return node
-
-    def visit_If(self, node):
-        raise DiffEquationError('Do not support "if" statement in differential equation.')
-
-    def visit_IfExp(self, node):
-        raise DiffEquationError('Do not support "if" expression in differential equation.')
-
-    def visit_For(self, node):
-        raise DiffEquationError('Do not support "for" loop in differential equation.')
-
-    def visit_While(self, node):
-        raise DiffEquationError('Do not support "while" loop in differential equation.')
-
-    def visit_Try(self, node):
-        raise DiffEquationError('Do not support "try" handler in differential equation.')
-
-    def visit_With(self, node):
-        raise DiffEquationError('Do not support "with" block in differential equation.')
-
-    def visit_Raise(self, node):
-        raise DiffEquationError('Do not support "raise" statement.')
-
-    def visit_Delete(self, node):
-        raise DiffEquationError('Do not support "del" operation.')
-
-
-def analyse_diff_eq(eq_code):
-    assert eq_code.strip() != ''
-    tree = ast.parse(eq_code)
-    analyser = DiffEquationAnalyser()
-    analyser.visit(tree)
-
-    res = DictPlus(variables=analyser.variables,
-                   expressions=analyser.expressions,
-                   returns=analyser.returns,
-                   return_type=analyser.return_type,
-                   f_expr=analyser.f_expr,
-                   g_expr=analyser.g_expr)
-    return res
 
 
 class FuncCallFinder(ast.NodeTransformer):
@@ -278,7 +159,7 @@ class FuncCallFinder(ast.NodeTransformer):
                 else:
                     s = ast2code(ast.fix_missing_locations(kv.value))
                     self.kwargs[kv.arg] = s.strip()
-            return ast.Name(f'_{self.name}_res')
+            return ast.Name('_res')
         else:
             args = [self.visit(arg) for arg in node.args]
             keywords = [self.visit(kv) for kv in node.keywords]
@@ -417,6 +298,7 @@ class CodeLineFormatter(ast.NodeTransformer):
         self.lefts = []
         self.rights = []
         self.lines = []
+        self.scope = dict()
 
     def visit_Assign(self, node, level=0):
         targets = node.targets
@@ -586,7 +468,9 @@ class LineFormatterForTrajectory(CodeLineFormatter):
             if target.value.id == 'ST' and target.slice.value.s in self.fixed_vars:
                 left = ast2code(ast.fix_missing_locations(target))
                 self.lefts.append(left)
-                self.lines.append(f'{prefix}{left} = {self.fixed_vars[target.slice.value.s]}')
+                key = target.slice.value.s
+                self.lines.append(f'{prefix}{left} = _fixed_{key}')
+                self.scope[f'_fixed_{key}'] = self.fixed_vars[key]
                 return node
 
         elif hasattr(target, 'elts'):
@@ -596,7 +480,9 @@ class LineFormatterForTrajectory(CodeLineFormatter):
                     if elt.value.id == 'ST' and elt.slice.value.s in self.fixed_vars:
                         left = ast2code(ast.fix_missing_locations(elt))
                         self.lefts.append(left)
-                        self.lines.append(f'{prefix}{left} = {self.fixed_vars[elt.slice.value.s]}')
+                        key = elt.slice.value.s
+                        self.lines.append(f'{prefix}{left} = _fixed_{key}')
+                        self.scope[f'_fixed_{key}'] = self.fixed_vars[key]
                         return node
                 left = ast2code(ast.fix_missing_locations(elt))
                 expr = ast2code(ast.fix_missing_locations(node.value))
@@ -609,7 +495,10 @@ class LineFormatterForTrajectory(CodeLineFormatter):
                     if isinstance(elt, ast.Subscript):
                         if elt.value.id == 'ST' and elt.slice.value.s in self.fixed_vars:
                             left = ast2code(ast.fix_missing_locations(elt))
-                            append_lines.append(f'{prefix}{left} = {self.fixed_vars[elt.slice.value.s]}')
+                            key = elt.slice.value.s
+                            line = f'{prefix}{left} = _fixed_{key}'
+                            self.scope[f'_fixed_{key}'] = self.fixed_vars[key]
+                            append_lines.append(line)
                 left = ast2code(ast.fix_missing_locations(target))
                 expr = ast2code(ast.fix_missing_locations(node.value))
                 self.lefts.append(target)
@@ -631,7 +520,9 @@ class LineFormatterForTrajectory(CodeLineFormatter):
             if node.target.value.id == 'ST' and node.target.slice.value.s in self.fixed_vars:
                 left = ast2code(ast.fix_missing_locations(node.target))
                 self.lefts.append(left)
-                self.lines.append(f'{prefix}{left} = {self.fixed_vars[node.target.slice.value.s]}')
+                key = node.target.slice.value.s
+                self.lines.append(f'{prefix}{left} = _fixed_{key}')
+                self.scope[f'_fixed_{key}'] = self.fixed_vars[key]
                 return node
 
         op = ast2code(ast.fix_missing_locations(node.op))
@@ -714,7 +605,8 @@ def word_replace(expr, substitutions):
     banana*_b+c5+8+func(A)
     """
     for var, replace_var in substitutions.items():
-        expr = re.sub(r'\b' + var + r'\b', str(replace_var), expr)
+        # expr = re.sub(r'\b' + var + r'\b', str(replace_var), expr)
+        expr = re.sub(r'\b(?<!\.)' + var + r'\b(?!\.)', str(replace_var), expr)
     return expr
 
 
